# Sales Automation Pipeline Bot

## About the project

### Features

* User can provide only list with desired products, bot will find these products and create table with all necessary
  information
* Excel and PDF output formats will be supported
* There is no need to enter all information in the table manually
* User can change currency exchange rates and price margin

### Main used technologies

`Python` is high-level programming language. Our team will use it for the back-end, and the tools listed here are

`aiogram` is a modern, fully asynchonous framework for the Telegram Bot API. It's needed to implement the Telegram Bot
back-end.

`MongoDB`  to store the products availability for the warehouse.

`pandas` is a Python library for operating with DataFrames effectively. It's needed to properly store and manipulate the
data from the PostgreSQL in runtime.

`LangChain` and LangGraph are frameworks allowing to build apps using LLMs. Thery are needed to process the user prompt,
parse the products, and merge the requirements in a commercial offer.

`Satu.kz API` is needed to get the information about the products.

`exchangeratesapi.io API` API for update currency exchange rates

`FastAPI` is a Python library for requesting data from the Internet. It's needed to operate with the Satu.kz and GPT
API's.

`XlsxWriter` is a Python library for creating Excel files. It's needed to form the final commercial offer as an Excel
file.

`borb` is a Python library for converting Excel files to PDF. It's needed to preview the commercial offer for the user
communicating with the bot.

# Quick start

### Clone directory

```git clone https://gitlab.pg.innopolis.university/r.nurgaliev/automated-commercial-offers.git```

Go to cloned directory, make virtual environment, activate it and install requirements

```cd src```

```python -m venv env``` 

```env\Scripts\activate```

```pip install -r requirements.txt```

Setup (or create) .env

**You can request TG token from project developers**

# Launch API from /src

```python -m API.api_main```

### Optional field - IP address for app host. 
```python -m API.api_main 0.0.0.0```   

this will launch app on **0.0.0.0** address

## .env file is required

# AND You can use Docker to launch app:

`cd src`  
`docker build -t aco-project . `  
`docker run -p 8000:8000 aco-project
`


# Main api calls:

`/satu_product/{sku}`
Get the product info with given SKU


`/create_pdf_offer`
Create PDF with specified products

Now - returns Commercial Offer file as response

#### Json in body is required for PDF generating request!

`{
"customer_info" : {"phone": "89977777777", "name": "aaaa"},  
"products": [
     {
        "sku": "2007045",
        "quantity": "3"
    },{
        "sku": "2363010",
        "quantity": "4"
    }
]}`


# Testing

Currently work in progress, launch from console supported

`pytest test/test_product_service.py
`  
`pytest test/test_satu_products.py
`  
For API we have total 11 used functions which could be tested
current state : 6 of them are covered with tests


