import os
from dotenv import load_dotenv
import motor.motor_asyncio

load_dotenv()

client = motor.motor_asyncio.AsyncIOMotorClient(os.getenv("MONGODB_URL"))
db = client.aco
aco_products = db.get_collection("aco_products")

# TODO add CRUD


def fill_db():
    from pydantic import ValidationError
    from typing import List
    from .schemas import ProductData
    import json
    import requests

    file_path = "data/product_quantity_OBO_data.json"

    with open(file_path, "r", encoding="utf-8") as f:
        data_list = json.load(f)

    valid_products: List[ProductData] = []

    url = os.getenv("MARKETPLACE_API_URL")
    header = {"x-api-key": os.getenv("MARKETPLACE_API_KEY")}

    for data in data_list:
        try:
            response = requests.post(
                url,
                headers=header,
                json={"sku_list": [data["code"]], "minimalist_mode": "True"},
            )
            response.raise_for_status()  # Raises an error for 4xx or 5xx status codes

            response_data = response.json()
            product = ProductData(**response_data[0])
            updated_product = product.model_copy(
                update={"quantity_in_stock": data["quantity_in_stock"]}
            )

            valid_products.append(updated_product)

            print("added product's data from SATU")
        except requests.exceptions.RequestException as e:
            product = ProductData(**data)
            valid_products.append(product)
            print("added product's data from managers Exel")
        except ValidationError as e:
            print(
                f"Validation error for item with sku {data.get('sku', 'unknown')}: {e}"
            )

    for product in valid_products:

        aco_products.insert_one(
            product.model_dump(by_alias=True, exclude=["id"])
        )


if __name__ == "__main__":
    fill_db()
