from __future__ import annotations

import os
from dotenv import load_dotenv

import requests

from datetime import datetime

load_dotenv()


class RatesModel:
    def __init__(self, rub_in_kzt, eur_in_kzt):
        self.rub_in_kzt = rub_in_kzt
        self.eur_in_kzt = eur_in_kzt


class ExchangeRatesService:
    def __init__(self):
        self._rates = None
        self._last_fetch = None

    def _url(self):
        key = os.getenv("EXCHANGE_RATES_KEY")
        return f"https://openexchangerates.org/api/latest.json?app_id={key}&symbols=rub,kzt,eur"

    def _parse(self, res):
        rates = res["rates"]
        tenge = rates["KZT"]
        euro = rates["EUR"]
        ruble = rates["RUB"]
        return RatesModel(tenge / ruble, tenge / euro)

    def _load_rates(self) -> None | RatesModel:
        url = self._url()
        headers = {"accept": "application/json"}
        res = requests.get(url, headers=headers)
        try:
            self._rates = self._parse(res.json())
            self._last_fetch = datetime.now()
            return self._rates
        except:
            return None

    def get_rates(self) -> None | RatesModel:
        # None on the first fetch
        if self._last_fetch is None or self._rates is None:
            return self._load_rates()
        delta = datetime.now() - self._last_fetch
        # Check more than an hour passed
        if delta.total_seconds() > 3600:
            return self._load_rates()
        # If data is fresh, use cache
        return self._rates


if __name__ == "__main__":

    exchange_rates_service = ExchangeRatesService()

    a = exchange_rates_service.get_rates()
