import json
import os

import numpy as np
from fastapi import FastAPI, HTTPException
import pandas as pd
import requests


class ProductsService:

    def get_product_info(self, sku_list: list):

        product_quantity_path = os.getenv("PRODUCT_QUANTITY_PATH")
        if __name__ == "__main__":
            with open(
                "../data/product_quantity_OBO_data.json", "r", encoding="utf-8"
            ) as file:
                products = json.load(file)
        else:

            with open(product_quantity_path, "r", encoding="utf-8") as file:
                products = json.load(file)

        product_info_list = []

        for item in sku_list:

            for pr in products:

                if int(pr["code"]) == int(item):
                    try:
                        cost = round(pr["cost"], 2)
                    except:
                        cost = pr["cost"]
                    dct = {
                        "sku": pr["code"],
                        "measure_unit": pr["unit"],
                        "cost": cost,
                        "quantity_in_stock": pr["quantity_in_stock"],
                    }

                    product_info_list.append(dct)

        return product_info_list

    def parse_data_for_co(self, products):

        url = os.getenv("MARKETPLACE_API_URL")
        header = {"x-api-key": os.getenv("MARKETPLACE_API_KEY")}

        product_quantity = {}
        for entry in products:
            id_key = entry["sku"]
            value_key = int(entry["quantity"])

            # Проверяем, был ли id_key уже прочитан
            if id_key in product_quantity:
                # Если да, складываем значение
                product_quantity[id_key] += value_key
            else:
                # Если нет, инициализируем его
                product_quantity[id_key] = value_key

        sku_list = [item for item in product_quantity]

        products = list(product_quantity.items())

        data = {"sku_list": sku_list, "minimalist_mode": "True"}

        response = requests.post(url, headers=header, json=data)

        satu_info = [
            {
                "sku": item["sku"],
                "currency": item["currency"],
                "name": item["name"],
            }
            for item in response.json()
        ]

        files_info = [
            {
                "sku": item["sku"],
                "measure_unit": item["measure_unit"],
                "cost": item["cost"],
                "quantity_in_stock": (
                    str(item["quantity_in_stock"])
                    if item["quantity_in_stock"] is not None
                    else "1"
                ),
            }
            for item in self.get_product_info(sku_list)
        ]

        return satu_info, files_info, products

    def build_co_dataframe(self, satu_info, files_info, products, ex_rates):

        df1 = pd.DataFrame(satu_info)
        df2 = pd.DataFrame(files_info)
        df3 = pd.DataFrame(products, columns=["sku", "quantity"])

        # print(f"len(df1): {len(df1)}")
        # print(f"len(df2): {len(df2)}")
        # print(f"len(df3): {len(df3)}")

        # TODO duplicates validation in structure

        # Выполняем объединение по ключу 'id'
        merged_df = pd.merge(df1, df2, on="sku", how="outer")
        merged_df = merged_df.dropna()
        merged_df = pd.merge(merged_df, df3, on="sku", how="outer")
        merged_df = merged_df.dropna()

        # pd.set_option('display.max_columns', None)

        commercial_offer_df = merged_df

        # print(commercial_offer_df['currency'])

        exchange_rates = {
            "EUR": round(ex_rates.eur_in_kzt, 2),
            "RUB": round(ex_rates.rub_in_kzt, 2),
        }

        # print(commercial_offer_df.columns)
        print(
            "INFO:commercial_offer positions: " + str(len(commercial_offer_df))
        )

        commercial_offer_df["str_cost"] = np.where(
            commercial_offer_df["currency"] == "KZT",
            commercial_offer_df["cost"].astype(str) + " " + "KZT",
            round(
                (
                    commercial_offer_df["cost"]
                    * commercial_offer_df["currency"].map(exchange_rates)
                ),
                2,
            ).astype(str)
            + " "
            + "KZT",
        )

        commercial_offer_df["sum_cost"] = commercial_offer_df[
            "cost"
        ] * commercial_offer_df["quantity"].astype(int)

        commercial_offer_df["cost"] = commercial_offer_df["cost"]

        commercial_offer_df["str_sum_cost"] = np.where(
            commercial_offer_df["currency"] == "KZT",
            commercial_offer_df["sum_cost"].astype(str) + " " + "KZT",
            round(
                (
                    commercial_offer_df["sum_cost"]
                    * commercial_offer_df["currency"].map(exchange_rates)
                ),
                2,
            ).astype(str)
            + " "
            + "KZT",
        )

        commercial_offer_df = commercial_offer_df.reset_index(col_fill="index")
        commercial_offer_df["index"] += 1

        return commercial_offer_df

    def create_co_data(self, products, ex_rates):
        satu_info, files_info, products = self.parse_data_for_co(products)
        commercial_offer_df = self.build_co_dataframe(
            satu_info, files_info, products, ex_rates
        )

        return commercial_offer_df
