from fastapi import APIRouter, Body, Depends, HTTPException, status
from ..schemas import ProductData
from ..database import aco_products
from bson import ObjectId

router = APIRouter(prefix="/aco_products", tags=["ACO Products"])


@router.post(
    "",
    response_description="Add product to CO",
    response_model=ProductData,
    status_code=status.HTTP_201_CREATED,
    response_model_by_alias=False,
)
async def create_aco_product(product: ProductData = Body(...)):
    """
    Insert a new aco_product record.

    A unique `id` will be created and provided in the response.
    """

    # TODO check for duplicates

    new_product = await aco_products.insert_one(
        product.model_dump(by_alias=True, exclude=["id"])
    )
    created_product = await aco_products.find_one(
        {"_id": new_product.inserted_id}
    )
    return created_product


@router.get(
    "/{sku}",
    response_description="Get a single aco-product by sku",
    response_model=ProductData,
    response_model_by_alias=False,
)
async def show_aco_product(sku: str):
    """
    Get the record for a specific product, looked up by `sku`.
    """
    if (product := await aco_products.find_one({"sku": sku})) is not None:
        return product

    raise HTTPException(status_code=404, detail=f"aco_product {sku} not found")
