import os
from dotenv import load_dotenv

import pandas as pd
from fastapi import APIRouter

from fastapi.responses import FileResponse
from API.products_service import ProductsService
from API.exchange_rates_service import ExchangeRatesService

from pdf_generator import pdf

load_dotenv()

router = APIRouter(prefix="/create_pdf_offer", tags=["PDF creation"])

PDF_OUTPUT = os.getenv("PDF_OUTPUT")

# url = "https://example.com/api"

products_service = ProductsService()
exchange_rates_service = ExchangeRatesService()

"""

Create PDF from custom list of products with ALL required parameters set manually. 

Body example:

[
    {
        "sku": "350012",
        "name": "Припой Castolin RB 5287 д.2,0мм, упак.15 прутков",
        "currency": "KZT",
        "measure_unit": "#Н/Д",
        "cost": 150,
        "quantity": "5",
        "sum_cost": 7.5,
        "str_cost": "150,00 KZT",
        "str_sum_cost": "7,50 KZT",
        "quantity_in_stock": "#Н/Д"
    },
    {
        "sku": "350013",
        "name": "Припой Castolin RB 5288 д.2,0мм, упак.15 прутков",
        "currency": "KZT",
        "measure_unit": "#Н/Д",
        "cost": 200,
        "quantity": "3",
        "sum_cost": 6,
        "str_cost": "200,00 KZT",
        "str_sum_cost": "6,00 KZT",
        "quantity_in_stock": "#Н/Д"
    }
]
"""


@router.post("/custom")
async def create_pdf_custom(products: list[dict]):
    products = pd.DataFrame(products)

    products = products.reset_index(col_fill="index")
    products["index"] += 1

    pdf.convert_to_pdf_and_save_(products)

    return FileResponse(
        PDF_OUTPUT, media_type="application/pdf", filename=PDF_OUTPUT
    )


"""

Create PDF from list of SKU's and quantity of each product. 
Another parameters wil be counted automatically.

Body example:

{
    "customer_info" : {"phone": "89977777777", "name": "aaaa"},
    "products": [
        {
            "sku": "2363010",
            "quantity": "11"
        },
        {
            "sku": "2007045",
            "quantity": "3"
            
        }
    ]
}
"""


@router.post("")
async def create_pdf(data: dict):
    ex_rates = exchange_rates_service.get_rates()

    products = data["products"]
    data_for_co = products_service.create_co_data(products, ex_rates)

    customer_info = data["customer_info"]
    pdf.convert_to_pdf_and_save_(data_for_co, customer_info)

    return FileResponse(
        PDF_OUTPUT, media_type="application/pdf", filename=PDF_OUTPUT
    )
