from __future__ import annotations

from pydantic import ConfigDict, BaseModel, Field
from pydantic.functional_validators import BeforeValidator
from typing import Optional
from collections import OrderedDict

from typing_extensions import Annotated

PyObjectId = Annotated[str, BeforeValidator(str)]


class ProductData(BaseModel):
    id: Optional[PyObjectId] = Field(alias="_id", default=None)
    sku: Optional[str] = None
    name: Optional[str] = None
    cost: Optional[float] = Field(alias="price", default=None)
    currency: Optional[str] = None
    measure_unit: Optional[str] = None
    pack_ratio: Optional[int] = None
    quantity_in_stock: Optional[int] = None
    model_config = ConfigDict(
        populate_by_name=True,
        arbitrary_types_allowed=True,
        json_schema_extra={
            "example": OrderedDict(
                [
                    ("sku", "Aртикул"),
                    ("name", "Наименование товара"),
                    ("cost", 12.23),
                    ("currency", "Валюта"),
                    ("measure_unit", "Единица измерения"),
                    ("pack_ratio", 10),
                    ("quantity_in_stock", 100),
                ]
            )
        },
    )
