import os
import sys
import pytest
from unittest.mock import patch

from fastapi.testclient import TestClient

parent_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
sys.path.insert(0, parent_dir)

from API.api_main import app


@pytest.fixture(scope="module")
def test_client():
    client = TestClient(app)
    yield client


@pytest.fixture
def mock_env_variables():
    os.environ["MARKETPLACE_API_URL"] = "http://mock.api/url"
    os.environ["MARKETPLACE_API_KEY"] = "mock_api_key"


@patch("requests.post")
def test_get_product_info(mock_post, test_client):
    sku = "7007063"

    mock_post.return_value.json.return_value = [
        {
            "sku": "7007063",
            "name": "Вертикальный угол 90° внешний 60x200 мм",
            "price": 37.81,
            "currency": "EUR",
            "measure_unit": "шт.",
            "presence": "available",
            "group": {
                "name": "Вертикальные угловые секции, резьбовое соединение"
            },
            "quantity_in_stock": None,
        }
    ]

    mock_post.return_value.status_code = 200

    response = test_client.get(f"/satu_product/{sku}")

    assert response.status_code == 200

    expected_response = {
        "product_data": [
            {
                "sku": "7007063",
                "name": "Вертикальный угол 90° внешний 60x200 мм",
                "price": 37.81,
                "currency": "EUR",
                "measure_unit": "шт.",
                "presence": "available",
                "group": {
                    "name": "Вертикальные угловые секции, резьбовое соединение"
                },
                "quantity_in_stock": None,
            }
        ]
    }

    assert response.json() == expected_response


@patch("requests.post")
def test_get_product_info_error(mock_post, test_client):
    sku = "99999999"

    # Настраиваем mock-объект для имитации ошибки API
    mock_post.return_value.status_code = 500

    response = test_client.get(f"/satu_product/{sku}")
    expected_error_response = {"product_data": {}}

    # Проверяем статус-код ответа
    assert response.json() == expected_error_response
