import os

import requests
from pydantic import BaseModel
from typing import Optional, List, Dict
from dotenv import load_dotenv

load_dotenv()


class GPTQuery(BaseModel):
    user_message: str
    system_message: str = (
        "You are an advanced AI assistant designed to extract relevant data from user messages. When a user provides input, identify and extract the values for 'sku', 'quantity', and 'cost'. Format the response as a JSON object with the following structure: { 'sku': <string>, 'cost': <float>, 'quantity': <int> }. If any field is not present in the user's message, return it with a null value."
    )
    json_only: bool = True


def query_gpt_api(query: GPTQuery):
    url = os.getenv("MARKETPLACE_GPT_URL")
    headers = {
        "x-api-key": os.getenv("MARKETPLACE_API_KEY"),
        "Content-Type": "application/json",
    }

    # Convert the Pydantic model to a dictionary for the request body
    data = query.model_dump()

    # Define headers including the x-api-key

    try:
        # Send POST request with the JSON body and headers
        response = requests.post(url, json=data, headers=headers)

        # Check if the response is successful
        if response.status_code == 200:
            return response.json()  # Assuming the server returns JSON response
        else:
            return f"Error: {response.status_code}, {response.text}"
    except Exception as e:
        return f"An error occurred: {str(e)}"
