from aiogram import F, Router
from aiogram.filters import CommandStart, Command

from aiogram.types import Message, FSInputFile, ReplyKeyboardRemove

from aiogram.fsm.state import StatesGroup, State
from aiogram.fsm.context import FSMContext

from aiogram.utils.keyboard import ReplyKeyboardMarkup, KeyboardButton
from aiogram.enums import ParseMode

from aiogram import Bot

from .gpt_api import GPTQuery, query_gpt_api
from API.database import aco_products
import json

from pdf_generator import pdf

from API.products_service import ProductsService
from API.exchange_rates_service import ExchangeRatesService

products_service = ProductsService()
exchange_rates_service = ExchangeRatesService()

import pandas as pd

router = Router()


# Создаем классы состояний для добавления товаров
class OfferCreation(StatesGroup):
    item_description = State()
    final_state = State()


user_items = []  # temp
aco_items = pd.DataFrame()  # TODO подумать над сохранением промежуточного КП


@router.message(CommandStart())
async def cmd_start(message: Message):
    # Спрашиваем, хочет ли пользователь начать
    markup = ReplyKeyboardMarkup(
        keyboard=[[KeyboardButton(text="Начать")]], resize_keyboard=True
    )

    await message.answer(
        f"Привет!\nЯ бот, который поможет составить КП\nНачнём?",
        reply_markup=markup,
    )


@router.message(Command("pdf"))
async def get_document(message: Message, bot: Bot):

    if not aco_items.empty:

        ex_rates = exchange_rates_service.get_rates()
        products = aco_items.to_dict(orient="records")

        # print(json.dumps(products, indent=4))

        data_for_co = products_service.create_co_data(products, ex_rates)

        # print(data_for_co.to_json(orient="records", indent=4))

        customer_info = {"phone": "89977777777", "name": "Nursultan"}
        pdf.convert_to_pdf_and_save_(data_for_co, customer_info)

        document = FSInputFile(path="data/file.pdf")
        await bot.send_document(message.chat.id, document=document, caption="")

    else:
        await message.answer("Документ пуст!")


@router.message(F.text == "Начать")
async def set_item(message: Message, state: FSMContext):
    global aco_items
    aco_items = pd.DataFrame(columns=["sku", "quantity"])

    await state.clear()

    await state.set_state(OfferCreation.item_description)
    await message.answer(
        "Введите информацию о товаре", reply_markup=ReplyKeyboardRemove()
    )


# Точка выхода: команда /done завершает процесс
@router.message(F.text == "Завершить")
async def finish_adding(message: Message, state: FSMContext):
    if not aco_items.empty:
        await message.answer(
            "Товары успешно добавлены.", reply_markup=ReplyKeyboardRemove()
        )
    else:
        await message.answer(
            "Вы не добавили ни одного товара.",
            reply_markup=ReplyKeyboardRemove(),
        )

    # Очищаем состояние и список
    await state.clear()
    user_items.clear()


# Обработчик выхода из состояния (например, команда /cancel)
@router.message(Command("cancel"))
async def cancel_adding(message: Message, state: FSMContext):
    await state.clear()
    user_items.clear()
    await message.answer("Процесс добавления товаров отменен.")


@router.message(OfferCreation.item_description)
async def item_description_entered(message: Message, state: FSMContext):

    item_description = message.text  # promt

    query = GPTQuery(user_message=item_description)

    response = query_gpt_api(query)
    content = json.loads(response["choices"][0]["message"]["content"])

    print(f"GPT RESPONSE: {content}")

    if (
        product := await aco_products.find_one({"sku": content["sku"]})
    ) is not None:

        global aco_items
        aco_items.loc[len(aco_items)] = [product["sku"], content["quantity"]]

        print(aco_items)

        # Добавляем товар в список
        user_items.append({"description": item_description})

    else:
        await message.answer("Tовар не найден")

    # Спрашиваем, хочет ли пользователь добавить еще один товар или завершить
    markup = ReplyKeyboardMarkup(
        keyboard=[[KeyboardButton(text="Завершить")]], resize_keyboard=True
    )

    await message.answer(
        f"Введите описание следующего товара.", reply_markup=markup
    )


@router.message(Command("help"))
async def cmd_start_2(message: Message):
    await message.answer("Help with bot")


@router.message(Command("settings"))
async def cmd_start_2(message: Message):
    await message.answer("Show current settings")
